const OracleService = require('../service/OracleService');

class updaterModel {
  constructor() {
    /**
     * UPDATER
     */
    // SELECT * FROM UPDATER UPR INNER JOIN
    // UPDATERSISTEMAS UPS ON (UPR.IDUPDATER = UPS.IDUPDATER) INNER JOIN
    // UPDATERVERSAO UPV ON (UPS.IDUPDATERSISTEMAS = UPV.IDUPDATERSISTEMAS)
    this.GET_UPDATER = `
      SELECT * FROM UPDATER UPR INNER JOIN
        UPDATERSISTEMAS UPS ON (UPR.IDUPDATER = UPS.IDUPDATER)
    `;

    /**
     * UPDATERSISTEMAS
     */
    this.GET_UPDATERSISTEMAS = `
     SELECT * FROM UPDATERSISTEMAS
      WHERE NOME = :NOME
   `;

    /**
     * UPDATERVERSAO
     */
    this.GET_UPDATERVERSAO = `
     SELECT * FROM UPDATERVERSAO
      WHERE IDUPDATERSISTEMAS = :IDUPDATERSISTEMAS
   `;

    this.SET_UPDATERVERSAO = `
      INSERT INTO UPDATERVERSAO
        (IDUPDATERSISTEMAS, CDOPERADOR, UPVERDATA, UPVERSTATUS)
        VALUES (:IDUPDATERSISTEMAS, :CDOPERADOR, :UPVERDATA, :UPVERSTATUS)
   `;

    /**
     * Histórico
     */
    this.GET_UPDATER_HISTORICO = `
      SELECT OP.NMOPERADOR, OP.CDOPERADOR, UPR.*, UPS.*, UPV.*
        FROM UPDATER UPR INNER JOIN
          UPDATERSISTEMAS UPS ON (UPR.IDUPDATER = UPS.IDUPDATER) INNER JOIN
          UPDATERVERSAO UPV ON (UPS.IDUPDATERSISTEMAS = UPV.IDUPDATERSISTEMAS) INNER JOIN
          OPERADOR OP ON (UPV.CDOPERADOR = OP.CDOPERADOR)
    `;
    /**
     * Histórico
     */
    this.GET_UPDATER_V_ATUAL = `
      SELECT OP.NMOPERADOR, OP.CDOPERADOR, UPR.*, UPS.*, UPV.*
        FROM UPDATER UPR INNER JOIN
          UPDATERSISTEMAS UPS ON (UPR.IDUPDATER = UPS.IDUPDATER) INNER JOIN
          UPDATERVERSAO UPV ON (UPS.IDUPDATERSISTEMAS = UPV.IDUPDATERSISTEMAS) INNER JOIN
          OPERADOR OP ON (UPV.CDOPERADOR = OP.CDOPERADOR)

        WHERE
          UPS.IDUPDATERSISTEMAS = :IDUPDATERSISTEMAS AND
          ROWNUM <= :MAX_ROW
        ORDER BY UPV.IDUPDATERVERSAO DESC
   `;

    this.GET_UPDATER_V_ALL = `
      SELECT OP.NMOPERADOR, OP.CDOPERADOR, UPR.*, UPS.*, UPV.*
        FROM UPDATER UPR INNER JOIN
          UPDATERSISTEMAS UPS ON (UPR.IDUPDATER = UPS.IDUPDATER) INNER JOIN
          UPDATERVERSAO UPV ON (UPS.IDUPDATERSISTEMAS = UPV.IDUPDATERSISTEMAS) INNER JOIN
          OPERADOR OP ON (UPV.CDOPERADOR = OP.CDOPERADOR)

        WHERE
          UPS.IDUPDATERSISTEMAS = :IDUPDATERSISTEMAS AND
          UPR.CDFILIAL = :CDFILIAL AND
          UPR.IDUPDATER = :IDUPDATER AND
          ROWNUM <= :MAX_ROW
        ORDER BY UPV.IDUPDATERVERSAO DESC
   `;

   /**
    * Retorna os sistemas a serem atualizados para cilial cdFilial
    */
   this.GET_UPDATER_ESPERA_ALL = `
    SELECT UPS.APLICACAO, UPV.IDUPDATERVERSAO
      FROM
        UPDATER UPR INNER JOIN
        UPDATERSISTEMAS UPS ON (UPR.IDUPDATER = UPS.IDUPDATER) INNER JOIN
        UPDATERVERSAO UPV ON (UPS.IDUPDATERSISTEMAS = UPV.IDUPDATERSISTEMAS) INNER JOIN
        OPERADOR OP ON (UPV.CDOPERADOR = OP.CDOPERADOR)

      WHERE
        UPR.CDFILIAL = :cdfilial AND
        UPV.UPVERSTATUS = 0
      GROUP BY UPS.APLICACAO, UPV.IDUPDATERVERSAO
   `;

   this.UPDATE_UPDATER = `
      UPDATE UPDATERVERSAO
        SET UPVERSTATUS = :UPVERSTATUS
      WHERE IDUPDATERVERSAO = :IDUPDATERVERSAO
   `;
  }

  getUpdaterVersao() {
    return OracleService.connect().then(() => {
      return OracleService.execute(this.GET_UPDATER);
    });
  }

  getUpdaterVersaoAtual(params) {
    return OracleService.connect().then(() => {
      return OracleService.execute(this.GET_UPDATER_V_ATUAL, params);
    });
  }

  getUpdaterVersaoAll(params) {
    return OracleService.getNewConnection().then(() => {
      return OracleService.execute(this.GET_UPDATER_V_ALL, params);
    });
  }

  /**
   * @author Daniel Souza <daniel.souza@teknisa.com>
   * @param {*} params {cdfilial: 9999}
   * @returns Lista de Sistemas a serem atualizados na filial cdFilial
   */
  getVersaoDisponivel(params) {
    return OracleService.getNewConnection().then(() => {
      return OracleService.execute(this.GET_UPDATER_ESPERA_ALL, params);
    });
  }

  updateVersaoDisponivel(params) {
    return OracleService.getNewConnection().then(() => {
      return OracleService.execute(this.UPDATE_UPDATER, params);
    });
  }

  setUpdaterVersao(params) {
    return OracleService.connect().then(() => {
      return OracleService.execute(this.SET_UPDATERVERSAO, params);
    });
  }
}

module.exports = new updaterModel();
